import path from 'path';
import fs from 'fs';
import { pathToFileURL } from 'url';
import chalk from 'chalk';

const APIService = class {
  APIs = {};

  async init() {
    const apiPath = path.resolve('src/api');
    const folders = fs.readdirSync(apiPath);
    for (const folder of folders) {
      const curPath = path.join(apiPath, folder, 'index.js');
      await import(pathToFileURL(curPath).toString()).then(res => {
        this.APIs[res.default.name] = res.default.execute;
      });
    }
    return this;
  }

  get(api) {
    if (api in this.APIs) {
      return this.APIs[api];
    } else {
      console.log(`${chalk.red('Error:')} API method not found`);
    }
  }
};

export default APIService;
